:root {
  --background-color-1: #A6BFC9;
	--background-color-2: #FCFCFF;
	--text-color: #000000;
	--link-color-1: #191923 ;
	--link-color-2: #D64933;
	--border-color: #191923;
}

body { 
  margin:0px; 
  padding:5px; 
  background:var(--background-color-1);
  color:var(--text-color);
  font-family:Arial,Helvetica,sans-serif;
  overflow:hidden;
}

a:link { color:var(--link-color-1); text-decoration:none; }
a:visited { color:var(--link-color-1); text-decoration:none; }
a:hover { color:var(--link-color-2); }
a.active { color:var(--link-color-2); }

input.query {
	padding: 4px 8px;
	margin: 12px 4px;
	width: 600px;
}

select.features {
	padding: 4px 8px;
	margin: 12px 4px;
	width: 600px;
}

button.submit {
	padding: 4px 8px;
	margin: 12px 4px;
}

.main {
  width:100%;
  height:100%;
  overflow:hidden;
}

.navigation {
  padding:10pt;
  height:100%;
  line-height: 150%;
  float:left;
  position:fixed;
  overflow-y:auto;
}

.heading {
  padding:15px;
  position:absolute;
  left: 250px;
  right: 25px;
  height: 250px;
}

.content {
  padding:15px;
  position:absolute;
  top: 250px;
  left: 250px;
  bottom: 25px;
  right: 25px;
  overflow:auto;
  background:var(--background-color-2);
}

.navigation ul {
  margin: 0;
  padding: 0;
  margin-left: 20px;
  color: var(--text-color);
  font-weight: bold;
}

.navigation ul ul {
  font-weight: normal;
}

.content ul li {
  margin:10pt;
}

.info {
  display:none;
}

table, th, td {
  border:1px solid var(--border-color);
  border-spacing:0px;
}

th, td {
  padding:5px;
}

table th {
	background-color:var(--background-color-1);
	color:var(--link-color-1);
	font-weight: bold;
	cursor: default;
}

table.sortable th:not(.sorttable_sorted):not(.sorttable_sorted_reverse):not(.sorttable_nosort):after { 
	content: " \25B4\25BE" 
}
